local assets=
{
	Asset("ATLAS", "images/inventoryimages/arrow.xml"),
    Asset("IMAGE", "images/inventoryimages/arrow.tex"),
	Asset("ATLAS", "images/inventoryimages/goldarrow.xml"),
    Asset("IMAGE", "images/inventoryimages/goldarrow.tex"),
	Asset("ATLAS", "images/inventoryimages/moonstonearrow.xml"),
    Asset("IMAGE", "images/inventoryimages/moonstonearrow.tex"),
	Asset("ATLAS", "images/inventoryimages/firearrow.xml"),
    Asset("IMAGE", "images/inventoryimages/firearrow.tex"),
	Asset("ATLAS", "images/inventoryimages/icearrow.xml"),
    Asset("IMAGE", "images/inventoryimages/icearrow.tex"),
	Asset("ATLAS", "images/inventoryimages/thunderarrow.xml"),
    Asset("IMAGE", "images/inventoryimages/thunderarrow.tex"),
	Asset("ATLAS", "images/inventoryimages/dischargedthunderarrow.xml"),
    Asset("IMAGE", "images/inventoryimages/dischargedthunderarrow.tex"),
}
prefabs = {

}

local function commonarrowfn(arrowanim, tags) 
    local inst = CreateEntity()
    local trans = inst.entity:AddTransform()
    local anim = inst.entity:AddAnimState()
    local netw = inst.entity:AddNetwork()
	
    MakeInventoryPhysics(inst)
 
	inst:AddTag("projectile") -- Tag is not doing anything by itself. I can be called by other stuffs though.
	inst:AddTag("arrow")

	if tags ~= nil then
        for i, v in ipairs(tags) do
            inst:AddTag(v)
        end
    end
	
	anim:SetBank("arrow")
    anim:SetBuild("bow")
    anim:PlayAnimation(arrowanim, true)
	
 --The following section is suitable for a DST compatible prefab.
    inst.entity:SetPristine()
	
    if not TheWorld.ismastersim then	
        return inst
    end
----------------------------------------------------------------

	inst:AddComponent("weapon")	
	inst.components.weapon:SetDamage(0)
	inst:AddComponent("projectile")
	
    inst:AddComponent("inspectable")

    inst:AddComponent("inventoryitem")
	inst.components.inventoryitem:SetOnPickupFn(onpickup)
	inst.components.inventoryitem:SetOnPutInInventoryFn(onputininventory)
	
	
	inst:AddComponent("zupalexsrangedweapons")
	
	inst:AddComponent("stackable")
	
    inst:ListenForEvent("onthrown", onthrown_regular)	
	
    return inst
end

local function onhitarrow_fire(inst, attacker, target)
	target.SoundEmitter:PlaySound("dontstarve/wilson/blowdart_impact_fire")
	
--	print("I am shooting a Fire Arrow")
	
	if target.components.burnable then
        target.components.burnable:Ignite(nil, attacker)
    end
    if target.components.freezable then
        target.components.freezable:Unfreeze()
    end
    if target.components.health then
        target.components.health:DoFireDamage((TUNING.BOWDMG*(TUNING.FIREARROWDMGMOD/2.0)), attacker)
    end
end

local function onhitarrow_ice(inst, attacker, target)
--	print("I am shooting an Ice Arrow")
   
    if target.components.burnable then
        if target.components.burnable:IsBurning() then
            target.components.burnable:Extinguish()
        elseif target.components.burnable:IsSmoldering() then
            target.components.burnable:SmotherSmolder()
        end
    end

    if target.components.freezable then
        target.components.freezable:AddColdness(1)
        target.components.freezable:SpawnShatterFX()
    end
end

local function onhitarrow_thunder(inst, attacker, target)
	local lightningstrike = SpawnPrefab("lightning")
	lightningstrike.Transform:SetPosition(target.Transform:GetWorldPosition())
	
	if not target:HasTag("stunned") then
		target:AddTag("stunned")
		
		if TheWorld.ismastersim and target.stuneffect == nil and target.components.health and not target.components.health:IsDead() then
			local symboltofollow = nil
			local symboltofollow_x = 0
			local symboltofollow_y = -100
			local symboltofollow_z = 0.02
		
			symboltofollow = target.components.combat.hiteffectsymbol
		
			if (symboltofollow == "marker" or symboltofollow == nil) and target.components.burnable then
				for k, v in pairs(target.components.burnable.fxdata) do
					if v.follow ~= nil then
						symboltofollow = v.follow
						symboltofollow_x = v.x
						symboltofollow_y = v.y - 190
						symboltofollow_z = v.z
					end
				end
			end
		
		
			if symboltofollow ~= nil and symboltofollow ~= "marker" then
				target.stuneffect = SpawnPrefab("stuneffect")
				target.stuneffect.Transform:SetPosition(target:GetPosition():Get())
				target.stuneffect:SetFollowTarget(target, symboltofollow, symboltofollow_x, symboltofollow_y, symboltofollow_z)
				target:ListenForEvent("death", function()
													if target.stuneffect ~= nil then
														target.stuneffect:SetFollowTarget(nil)
														target.stuneffect = nil
													end
												end
									)	
			end
		end
		
		if target.components.locomotor then
			target.preventmoving = target:DoPeriodicTask(0, function(target) target.components.locomotor:Stop() end)
			target.electricstun = target:DoPeriodicTask(4, function(target) 
																	target:RemoveTag("stunned")
																	
																	if TheWorld.ismastersim and target.stuneffect  ~= nil then
																		target.stuneffect:SetFollowTarget(nil)
																		target.stuneffect = nil
																	end
																	
																	if target.electricstun then
																		target.electricstun:Cancel()
																		target.electricstun = nil
																	end
																
																	if target.preventmoving then
																		target.preventmoving:Cancel()
																		target.preventmoving = nil
																	end
																end
															)															
		end
	end
	
	if TheWorld.state.israining then
		if math.random() <= 0.5 then
			if attacker.components.playerlightningtarget then
				attacker.components.playerlightningtarget:DoStrike()
				TheWorld:PushEvent("ms_sendlightningstrike", attacker:GetPosition())
			end
		end
	end
end

local function shootarrow(inst)
    inst.AnimState:PlayAnimation("arrow_flight")
    inst:AddTag("NOCLICK")
    inst.persists = false
end

local function shootgoldarrow(inst)
    inst.AnimState:PlayAnimation("goldarrow_flight")
    inst:AddTag("NOCLICK")
    inst.persists = false
end

local function shootmoonstonearrow(inst)
    inst.AnimState:PlayAnimation("moonstonearrow_flight", true)
	inst.Light:Enable(true)
    inst:AddTag("NOCLICK")
    inst.persists = false
end

local function onhitarrow_healing(inst, attacker, target)
	if target ~= nil then
		if target.components.health ~= nil and not target.components.health:IsDead() then
		target.components.health:DoDelta(25)
		end
		
		if target.components.sanity ~= nil then
			target.components.sanity:DoDelta(-5)
		end
	end
end

local function regulararrowfn()
	local inst = commonarrowfn("arrow_idle", { "piercing", "sharp", "recoverable" })
	
	if not TheWorld.ismastersim then
        return inst
    end
	
	inst.Physics:SetCollisionCallback(oncollide)
	
	inst.components.projectile:SetSpeed(40)
	inst.components.projectile:SetOnThrownFn(shootarrow)
	inst.components.projectile:SetOnHitFn(onhitcommon)
	inst.components.projectile:SetOnMissFn(onmissarrow_regular)
	inst.components.projectile:SetLaunchOffset(Vector3(0.35, 1.05, 0))
	
	inst.components.inventoryitem.imagename = "arrow"
    inst.components.inventoryitem.atlasname = "images/inventoryimages/arrow.xml"
	
	inst.components.stackable.maxsize = TUNING.STACK_SIZE_MEDITEM
	
	return inst
end

local function goldarrowfn()
	local inst = commonarrowfn("goldarrow_idle", { "piercing", "sharp", "golden", "recoverable" })
	
	if not TheWorld.ismastersim then
        return inst
    end
	
	inst.Physics:SetCollisionCallback(oncollide)
	
	inst.components.projectile:SetSpeed(40)
	inst.components.projectile:SetOnThrownFn(shootgoldarrow)
	inst.components.projectile:SetOnHitFn(onhitcommon)
	inst.components.projectile:SetOnMissFn(onmissarrow_regular)
	inst.components.projectile:SetLaunchOffset(Vector3(0.35, 1.05, 0))
	
	inst.components.inventoryitem.imagename = "goldarrow"
    inst.components.inventoryitem.atlasname = "images/inventoryimages/goldarrow.xml"
	
	inst.components.stackable.maxsize = TUNING.STACK_SIZE_MEDITEM
	
	return inst
end

local function ondroppedMSarrow(inst)
	inst.Light:Enable(true)
end

local function moonstonearrowfn()
	local inst = commonarrowfn("moonstonearrow_idle", { "piercing", "sharp", "moonstone", "recoverable" })
	
	local light = inst.entity:AddLight()
	
	inst.Light:SetIntensity(0.6)
	inst.Light:SetRadius(0.5)
	inst.Light:SetFalloff(0.75)
	inst.Light:Enable(true)
	inst.Light:SetColour(0/255, 0/255, 255/255)
	
	if not TheWorld.ismastersim then
        return inst
    end
	
	inst.Physics:SetCollisionCallback(oncollide)
	
    inst.components.inventoryitem:SetOnDroppedFn(ondroppedMSarrow)
	
	inst.components.projectile:SetSpeed(40)
	inst.components.projectile:SetOnThrownFn(shootmoonstonearrow)
	inst.components.projectile:SetOnHitFn(onhitcommon)
	inst.components.projectile:SetOnMissFn(onmissarrow_regular)
	inst.components.projectile:SetLaunchOffset(Vector3(0.35, 1.05, 0))
	
	inst.components.inventoryitem.imagename = "moonstonearrow"
    inst.components.inventoryitem.atlasname = "images/inventoryimages/moonstonearrow.xml"
	
	inst.components.stackable.maxsize = TUNING.STACK_SIZE_MEDITEM
	
	return inst
end

local function onmissarrow_special(inst, attacker, target)
	inst:Remove()
end

local function shootarrow_fire(inst)
    inst.AnimState:PlayAnimation("firearrow_flight", true)
    inst:AddTag("NOCLICK")
    inst.persists = false
end

local function shootarrow_ice(inst)
    inst.AnimState:PlayAnimation("icearrow_flight", true)
    inst:AddTag("NOCLICK")
    inst.persists = false
end

local function firearrowfn()
	local inst = commonarrowfn("firearrow_idle", { "burning", "recoverable" })
	
	if not TheWorld.ismastersim then
        return inst
    end
	
	inst.Physics:SetCollisionCallback(oncollide)
		
	inst.components.zupalexsrangedweapons:SetSpecificOnHitfn(onhitarrow_fire)
	
	inst.components.projectile:SetSpeed(40)
	inst.components.projectile:SetOnThrownFn(shootarrow_fire)
    inst.components.projectile:SetOnHitFn(onhitcommon)
    inst.components.projectile:SetOnMissFn(onmissarrow_regular)
	inst.components.projectile:SetLaunchOffset(Vector3(0.35, 1.05, 0))
	
	inst.components.inventoryitem.imagename = "firearrow"
    inst.components.inventoryitem.atlasname = "images/inventoryimages/firearrow.xml"
	
	inst.components.stackable.maxsize = TUNING.STACK_SIZE_MEDITEM
	
	return inst
end

local function icearrowfn()
	local inst = commonarrowfn("icearrow_idle", { "freezing", "recoverable" })
	
	if not TheWorld.ismastersim then
        return inst
    end
	
	inst.Physics:SetCollisionCallback(oncollide)
	
	inst.components.zupalexsrangedweapons:SetSpecificOnHitfn(onhitarrow_ice)
	
	inst.components.projectile:SetSpeed(40)
	inst.components.projectile:SetOnThrownFn(shootarrow_ice)
    inst.components.projectile:SetOnHitFn(onhitcommon)
    inst.components.projectile:SetOnMissFn(onmissarrow_regular)
	inst.components.projectile:SetLaunchOffset(Vector3(0.35, 1.05, 0))
	
	inst.components.inventoryitem.imagename = "icearrow"
    inst.components.inventoryitem.atlasname = "images/inventoryimages/icearrow.xml"
	
	inst.components.stackable.maxsize = TUNING.STACK_SIZE_MEDITEM
	
	return inst
end

local function startflickering(inst)
	if inst.flickering == nil then
		inst.flickering = inst:DoPeriodicTask(0.075, function(inst)
														inst.lightstate = inst.Light:IsEnabled()
														inst.Light:Enable(not inst.lightstate)
													end
											)
	end
end

local function stopflickering(inst)
	if inst.flickering ~= nil then
		inst.flickering:Cancel()
		inst.flickering = nil
	end
end

local function shootarrow_thunder(inst)
    inst.AnimState:PlayAnimation("thunderarrow_flight", true)
	startflickering(inst)
    inst:AddTag("NOCLICK")
    inst.persists = false
end

local function thunderarrowfn()
	local inst = commonarrowfn("thunderarrow_idle", { "electric", "recoverable" })
	
	local light = inst.entity:AddLight()
	
	inst.Light:SetIntensity(0.4)
	inst.Light:SetRadius(0.3)
	inst.Light:SetFalloff(0.66)
	inst.Light:SetColour(152/255, 229/255, 243/255)
	inst.Light:Enable(false)
	
	if not TheWorld.ismastersim then
        return inst
    end
	
	inst.Physics:SetCollisionCallback(oncollide)
	
	inst.components.zupalexsrangedweapons:SetSpecificOnHitfn(onhitarrow_thunder)
		
	inst.components.inventoryitem:SetOnDroppedFn(startflickering)	
	inst.components.inventoryitem:SetOnPickupFn(stopflickering)
	
	inst.components.projectile:SetSpeed(40)
	inst.components.projectile:SetOnThrownFn(shootarrow_thunder)
    inst.components.projectile:SetOnHitFn(onhitcommon)
    inst.components.projectile:SetOnMissFn(onmissarrow_regular)
	inst.components.projectile:SetLaunchOffset(Vector3(0.35, 1.05, 0))
	
	inst.components.inventoryitem.imagename = "thunderarrow"
    inst.components.inventoryitem.atlasname = "images/inventoryimages/thunderarrow.xml"
	
	inst.components.stackable.maxsize = TUNING.STACK_SIZE_MEDITEM
	
	inst.components.zupalexsrangedweapons.stimuli = "electric"
	
	return inst
end

local function shootarrow_thunder_discharged(inst)
    inst.AnimState:PlayAnimation("thunderarrow_flight_discharged", true)
    inst:AddTag("NOCLICK")
    inst.persists = false
end

local function dischargedthunderarrowfn()
	local inst = commonarrowfn("thunderarrow_idle_discharged", { "electric", "discharged", "recoverable" })
	
	if not TheWorld.ismastersim then
        return inst
    end
	
	inst.Physics:SetCollisionCallback(oncollide)
	
	inst.components.projectile:SetSpeed(40)
	inst.components.projectile:SetOnThrownFn(shootarrow_thunder_discharged)
    inst.components.projectile:SetOnHitFn(onhitcommon)
    inst.components.projectile:SetOnMissFn(onmissarrow_regular)
	inst.components.projectile:SetLaunchOffset(Vector3(0.35, 1.05, 0))
	
	inst.components.inventoryitem.imagename = "dischargedthunderarrow"
    inst.components.inventoryitem.atlasname = "images/inventoryimages/dischargedthunderarrow.xml"
	
	inst.components.stackable.maxsize = TUNING.STACK_SIZE_MEDITEM
	
	return inst
end

--------------------------------------------------------------------------MAGIC PROJECTILES-------------------------------------------------------------------------

local function commonmagicprojfn(arrowanim, tags) 
    local inst = CreateEntity()
    local trans = inst.entity:AddTransform()
    local anim = inst.entity:AddAnimState()
    local netw = inst.entity:AddNetwork()
 
    MakeInventoryPhysics(inst)
 
 	anim:SetBank("magicprojectile")
    anim:SetBuild("bow")
    anim:PlayAnimation(arrowanim)
 
	inst:AddTag("projectile") -- Tag is not doing anything by itself. I can be called by other stuffs though.
	inst:AddTag("arrow")
	inst:AddTag("magic")

	if tags ~= nil then
        for i, v in ipairs(tags) do
            inst:AddTag(v)
        end
    end
	
 --The following section is suitable for a DST compatible prefab.
    inst.entity:SetPristine()
	
    if not TheWorld.ismastersim then	
        return inst
    end
----------------------------------------------------------------

	inst:AddComponent("weapon")
	inst.components.weapon:SetDamage(0)
	inst:AddComponent("projectile")	
	
	inst:AddComponent("zupalexsrangedweapons")
	
    inst:ListenForEvent("onthrown", onthrown_regular)
	
    return inst
end

local function shootshadowarrow(inst)
    inst.AnimState:PlayAnimation("shadowarrow_flight")
    inst:AddTag("NOCLICK")
    inst.persists = false
end

local function shadowarrowfn()
	local inst = commonmagicprojfn("shadowarrow_flight", { "shadow" })
	
	RemovePhysicsColliders(inst)
	
	local light = inst.entity:AddLight()
	
	inst.Light:SetIntensity(0.8)
	inst.Light:SetRadius(3)
	inst.Light:SetFalloff(0.33)
	inst.Light:Enable(true)
	inst.Light:SetColour(119/255, 45/255, 166/255)
	
	inst:AddTag("energy")
	inst:AddTag("nocollisionoverride")
	
	inst.persists = false
	
	if not TheWorld.ismastersim then
        return inst
    end
	
--	inst.Physics:SetCollisionCallback(oncollide)
	
	inst.components.projectile:SetSpeed(40)
	inst.components.projectile:SetOnThrownFn(shootshadowarrow)
	inst.components.projectile:SetOnHitFn(onhitcommon)
	inst.components.projectile:SetOnMissFn(inst.Remove)
	inst.components.projectile:SetLaunchOffset(Vector3(0.35, 1.05, 0))
	
	return inst
end

local function shootlightarrow(inst)
    inst.AnimState:PlayAnimation("lightarrow_flight")
    inst:AddTag("NOCLICK")
    inst.persists = false
end

local function lightarrowfn()
	local inst = commonmagicprojfn("lightarrow_flight", { "light" })
	
	RemovePhysicsColliders(inst)
	
	local light = inst.entity:AddLight()
	
	inst.Light:SetIntensity(0.8)
	inst.Light:SetRadius(3)
	inst.Light:SetFalloff(0.33)
	inst.Light:Enable(true)
	inst.Light:SetColour(255/255, 253/255, 54/255)
	
	inst:AddTag("energy")
	inst:AddTag("nocollisionoverride")
	
	inst.persists = false
	
	if not TheWorld.ismastersim then
        return inst
    end
	
--	inst.Physics:SetCollisionCallback(oncollide)
	
	inst.components.projectile:SetSpeed(30)
	inst.components.projectile:SetOnThrownFn(shootlightarrow)
	inst.components.projectile:SetOnHitFn(onhitcommon)
	inst.components.projectile:SetOnMissFn(inst.Remove)
	inst.components.projectile:SetLaunchOffset(Vector3(0.35, 1.05, 0))
	
	return inst
end

local function shoothealingarrow(inst)
    inst.AnimState:PlayAnimation("healingarrow_flight")
    inst:AddTag("NOCLICK")
    inst.persists = false
end

local function healingarrowfn()
	local inst = commonmagicprojfn("healingarrow_flight", { "healing" })
	
	RemovePhysicsColliders(inst)
	
	local light = inst.entity:AddLight()
	
	inst.Light:SetIntensity(0.8)
	inst.Light:SetRadius(3)
	inst.Light:SetFalloff(0.33)
	inst.Light:Enable(true)
	inst.Light:SetColour(247/255, 116/255, 255/255)
	
	inst:AddTag("energy")
	inst:AddTag("nocollisionoverride")
	
	inst.persists = false
	
	if not TheWorld.ismastersim then
        return inst
    end
	
--	inst.Physics:SetCollisionCallback(oncollide)
	
	inst.components.zupalexsrangedweapons:SetSpecificOnHitfn(onhitarrow_healing)
	
	inst.components.projectile:SetSpeed(20)
	inst.components.projectile:SetOnThrownFn(shoothealingarrow)
	inst.components.projectile:SetOnHitFn(onhitcommon)
	inst.components.projectile:SetOnMissFn(inst.Remove)
	inst.components.projectile:SetLaunchOffset(Vector3(0.35, 1.05, 0))
	
	return inst
end


return  Prefab("common/inventory/arrow", regulararrowfn, assets, prefabs),
		Prefab("common/inventory/goldarrow", goldarrowfn, assets, prefabs),
		Prefab("common/inventory/moonstonearrow", moonstonearrowfn, assets, prefabs),
		Prefab("common/inventory/firearrow", firearrowfn, assets, prefabs),
		Prefab("common/inventory/icearrow", icearrowfn, assets, prefabs),
		Prefab("common/inventory/thunderarrow", thunderarrowfn, assets, prefabs),
		Prefab("common/inventory/dischargedthunderarrow", dischargedthunderarrowfn, assets, prefabs),
		Prefab("common/inventory/shadowarrow", shadowarrowfn, assets),
		Prefab("common/inventory/lightarrow", lightarrowfn, assets),
		Prefab("common/inventory/healingarrow", healingarrowfn, assets)